@@ -1,34 +1,45 @@
 # sql/dml.py
-# Copyright (C) 2009-2017 the SQLAlchemy authors and contributors
+# Copyright (C) 2009-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 """
-Provide :class:`.Insert`, :class:`.Update` and :class:`.Delete`.
+Provide :class:`_expression.Insert`, :class:`_expression.Update` and
+:class:`_expression.Delete`.
 
 """
 
-from .base import Executable, _generative, _from_objects, DialectKWArgs, \
-    ColumnCollection
-from .elements import ClauseElement, _literal_as_text, Null, and_, _clone, \
-    _column_as_key
-from .selectable import _interpret_as_from, _interpret_as_select, \
-    HasPrefixes, HasCTE
-from .. import util
+from .base import _from_objects
+from .base import _generative
+from .base import DialectKWArgs
+from .base import Executable
+from .elements import _clone
+from .elements import _column_as_key
+from .elements import _literal_as_text
+from .elements import and_
+from .elements import ClauseElement
+from .elements import Null
+from .selectable import _interpret_as_from
+from .selectable import _interpret_as_select
+from .selectable import HasCTE
+from .selectable import HasPrefixes
 from .. import exc
+from .. import util
 
 
 class UpdateBase(
-        HasCTE, DialectKWArgs, HasPrefixes, Executable, ClauseElement):
+    HasCTE, DialectKWArgs, HasPrefixes, Executable, ClauseElement
+):
     """Form the base for ``INSERT``, ``UPDATE``, and ``DELETE`` statements.
 
     """
 
-    __visit_name__ = 'update_base'
+    __visit_name__ = "update_base"
 
-    _execution_options = \
-        Executable._execution_options.union({'autocommit': True})
+    _execution_options = Executable._execution_options.union(
+        {"autocommit": True}
+    )
     _hints = util.immutabledict()
     _parameter_ordering = None
     _prefixes = ()
@@ -37,30 +48,33 @@ class UpdateBase(
     def _process_colparams(self, parameters):
         def process_single(p):
             if isinstance(p, (list, tuple)):
-                return dict(
-                    (c.key, pval)
-                    for c, pval in zip(self.table.c, p)
-                )
+                return dict((c.key, pval) for c, pval in zip(self.table.c, p))
             else:
                 return p
 
         if self._preserve_parameter_order and parameters is not None:
-            if not isinstance(parameters, list) or \
-                    (parameters and not isinstance(parameters[0], tuple)):
+            if not isinstance(parameters, list) or (
+                parameters and not isinstance(parameters[0], tuple)
+            ):
                 raise ValueError(
                     "When preserve_parameter_order is True, "
-                    "values() only accepts a list of 2-tuples")
+                    "values() only accepts a list of 2-tuples"
+                )
             self._parameter_ordering = [key for key, value in parameters]
 
             return dict(parameters), False
 
-        if (isinstance(parameters, (list, tuple)) and parameters and
-                isinstance(parameters[0], (list, tuple, dict))):
+        if (
+            isinstance(parameters, (list, tuple))
+            and parameters
+            and isinstance(parameters[0], (list, tuple, dict))
+        ):
 
             if not self._supports_multi_parameters:
                 raise exc.InvalidRequestError(
                     "This construct does not support "
-                    "multiple parameter sets.")
+                    "multiple parameter sets."
+                )
 
             return [process_single(p) for p in parameters], True
         else:
@@ -77,17 +91,19 @@ class UpdateBase(
         raise NotImplementedError(
             "params() is not supported for INSERT/UPDATE/DELETE statements."
             " To set the values for an INSERT or UPDATE statement, use"
-            " stmt.values(**parameters).")
+            " stmt.values(**parameters)."
+        )
 
     def bind(self):
         """Return a 'bind' linked to this :class:`.UpdateBase`
-        or a :class:`.Table` associated with it.
+        or a :class:`_schema.Table` associated with it.
 
         """
         return self._bind or self.table.bind
 
     def _set_bind(self, bind):
         self._bind = bind
+
     bind = property(bind, _set_bind)
 
     @_generative
@@ -107,7 +123,8 @@ class UpdateBase(
 
         The given collection of column expressions should be derived from
         the table that is
-        the target of the INSERT, UPDATE, or DELETE.  While :class:`.Column`
+        the target of the INSERT, UPDATE, or DELETE.  While
+        :class:`_schema.Column`
         objects are typical, the elements can also be expressions::
 
             stmt = table.insert().returning(
@@ -121,7 +138,8 @@ class UpdateBase(
 
         Upon execution, the values of the columns to be returned are made
         available via the result set and can be iterated using
-        :meth:`.ResultProxy.fetchone` and similar.   For DBAPIs which do not
+        :meth:`_engine.ResultProxy.fetchone` and similar.
+        For DBAPIs which do not
         natively support returning values (i.e. cx_oracle), SQLAlchemy will
         approximate this behavior at the result level so that a reasonable
         amount of behavioral neutrality is provided.
@@ -158,9 +176,9 @@ class UpdateBase(
 
         The text of the hint is rendered in the appropriate
         location for the database backend in use, relative
-        to the :class:`.Table` that is the subject of this
+        to the :class:`_schema.Table` that is the subject of this
         statement, or optionally to that of the given
-        :class:`.Table` passed as the ``selectable`` argument.
+        :class:`_schema.Table` passed as the ``selectable`` argument.
 
         The ``dialect_name`` option will limit the rendering of a particular
         hint to a particular backend. Such as, to add a hint
@@ -168,28 +186,26 @@ class UpdateBase(
 
             mytable.insert().with_hint("WITH (PAGLOCK)", dialect_name="mssql")
 
-        .. versionadded:: 0.7.6
-
         :param text: Text of the hint.
-        :param selectable: optional :class:`.Table` that specifies
+        :param selectable: optional :class:`_schema.Table` that specifies
          an element of the FROM clause within an UPDATE or DELETE
          to be the subject of the hint - applies only to certain backends.
         :param dialect_name: defaults to ``*``, if specified as the name
          of a particular dialect, will apply these hints only when
          that dialect is in use.
-         """
+
+        """
         if selectable is None:
             selectable = self.table
 
-        self._hints = self._hints.union(
-            {(selectable, dialect_name): text})
+        self._hints = self._hints.union({(selectable, dialect_name): text})
 
 
 class ValuesBase(UpdateBase):
     """Supplies support for :meth:`.ValuesBase.values` to
     INSERT and UPDATE constructs."""
 
-    __visit_name__ = 'values_base'
+    __visit_name__ = "values_base"
 
     _supports_multi_parameters = False
     _has_multi_parameters = False
@@ -199,19 +215,21 @@ class ValuesBase(UpdateBase):
 
     def __init__(self, table, values, prefixes):
         self.table = _interpret_as_from(table)
-        self.parameters, self._has_multi_parameters = \
-            self._process_colparams(values)
+        self.parameters, self._has_multi_parameters = self._process_colparams(
+            values
+        )
         if prefixes:
             self._setup_prefixes(prefixes)
 
     @_generative
     def values(self, *args, **kwargs):
-        r"""specify a fixed VALUES clause for an INSERT statement, or the SET
+        r"""Specify a fixed VALUES clause for an INSERT statement, or the SET
         clause for an UPDATE.
 
-        Note that the :class:`.Insert` and :class:`.Update` constructs support
+        Note that the :class:`_expression.Insert` and
+        :class:`_expression.Update` constructs support
         per-execution time formatting of the VALUES and/or SET clauses,
-        based on the arguments passed to :meth:`.Connection.execute`.
+        based on the arguments passed to :meth:`_engine.Connection.execute`.
         However, the :meth:`.ValuesBase.values` method can be used to "fix" a
         particular set of parameters into the statement.
 
@@ -224,7 +242,8 @@ class ValuesBase(UpdateBase):
         onto the existing list of values.
 
         :param \**kwargs: key value pairs representing the string key
-          of a :class:`.Column` mapped to the value to be rendered into the
+          of a :class:`_schema.Column`
+          mapped to the value to be rendered into the
           VALUES or SET clause::
 
                 users.insert().values(name="some name")
@@ -235,28 +254,29 @@ class ValuesBase(UpdateBase):
          a dictionary, tuple, or list of dictionaries or tuples can be passed
          as a single positional argument in order to form the VALUES or
          SET clause of the statement.  The forms that are accepted vary
-         based on whether this is an :class:`.Insert` or an :class:`.Update`
-         construct.
+         based on whether this is an :class:`_expression.Insert` or an
+         :class:`_expression.Update` construct.
 
-         For either an :class:`.Insert` or :class:`.Update` construct, a
-         single dictionary can be passed, which works the same as that of
-         the kwargs form::
+         For either an :class:`_expression.Insert` or
+         :class:`_expression.Update`
+         construct, a single dictionary can be passed, which works the same as
+         that of the kwargs form::
 
             users.insert().values({"name": "some name"})
 
             users.update().values({"name": "some new name"})
 
-         Also for either form but more typically for the :class:`.Insert`
-         construct, a tuple that contains an entry for every column in the
-         table is also accepted::
+         Also for either form but more typically for the
+         :class:`_expression.Insert` construct, a tuple that contains an
+         entry for every column in the table is also accepted::
 
             users.insert().values((5, "some name"))
 
-         The :class:`.Insert` construct also supports being passed a list
-         of dictionaries or full-table-tuples, which on the server will
-         render the less common SQL syntax of "multiple values" - this
-         syntax is supported on backends such as SQLite, PostgreSQL, MySQL,
-         but not necessarily others::
+         The :class:`_expression.Insert` construct also supports being
+         passed a list of dictionaries or full-table-tuples, which on the
+         server will render the less common SQL syntax of "multiple values" -
+         this syntax is supported on backends such as SQLite, PostgreSQL,
+         MySQL, but not necessarily others::
 
             users.insert().values([
                                 {"name": "some name"},
@@ -275,7 +295,8 @@ class ValuesBase(UpdateBase):
          NOT the same as using traditional executemany() form**.  The above
          syntax is a **special** syntax not typically used.  To emit an
          INSERT statement against multiple rows, the normal method is
-         to pass a multiple values list to the :meth:`.Connection.execute`
+         to pass a multiple values list to the
+         :meth:`_engine.Connection.execute`
          method, which is supported by all database backends and is generally
          more efficient for a very large number of parameters.
 
@@ -287,11 +308,13 @@ class ValuesBase(UpdateBase):
 
            .. versionchanged:: 1.0.0 an INSERT that uses a multiple-VALUES
               clause, even a list of length one,
-              implies that the :paramref:`.Insert.inline` flag is set to
+              implies that the :paramref:`_expression.Insert.inline`
+              flag is set to
               True, indicating that the statement will not attempt to fetch
               the "last inserted primary key" or other defaults.  The
               statement deals with an arbitrary number of rows, so the
-              :attr:`.ResultProxy.inserted_primary_key` accessor does not
+              :attr:`_engine.ResultProxy.inserted_primary_key`
+              accessor does not
               apply.
 
            .. versionchanged:: 1.0.0 A multiple-VALUES INSERT now supports
@@ -300,24 +323,26 @@ class ValuesBase(UpdateBase):
               callable is invoked for each row.   See :ref:`bug_3288`
               for other details.
 
-         The :class:`.Update` construct supports a special form which is a
+         The :class:`_expression.Update`
+         construct supports a special form which is a
          list of 2-tuples, which when provided must be passed in conjunction
          with the
-         :paramref:`~sqlalchemy.sql.expression.update.preserve_parameter_order`
+         :paramref:`_expression.update.preserve_parameter_order`
          parameter.
          This form causes the UPDATE statement to render the SET clauses
-         using the order of parameters given to :meth:`.Update.values`, rather
-         than the ordering of columns given in the :class:`.Table`.
+         using the order of parameters given to
+         :meth:`_expression.Update.values`, rather
+         than the ordering of columns given in the :class:`_schema.Table`.
 
            .. versionadded:: 1.0.10 - added support for parameter-ordered
               UPDATE statements via the
-              :paramref:`~sqlalchemy.sql.expression.update.preserve_parameter_order`
+              :paramref:`_expression.update.preserve_parameter_order`
               flag.
 
            .. seealso::
 
               :ref:`updates_order_parameters` - full example of the
-              :paramref:`~sqlalchemy.sql.expression.update.preserve_parameter_order`
+              :paramref:`_expression.update.preserve_parameter_order`
               flag
 
         .. seealso::
@@ -325,30 +350,35 @@ class ValuesBase(UpdateBase):
             :ref:`inserts_and_updates` - SQL Expression
             Language Tutorial
 
-            :func:`~.expression.insert` - produce an ``INSERT`` statement
+            :func:`_expression.insert` - produce an ``INSERT`` statement
 
-            :func:`~.expression.update` - produce an ``UPDATE`` statement
+            :func:`_expression.update` - produce an ``UPDATE`` statement
 
         """
         if self.select is not None:
             raise exc.InvalidRequestError(
-                "This construct already inserts from a SELECT")
+                "This construct already inserts from a SELECT"
+            )
         if self._has_multi_parameters and kwargs:
             raise exc.InvalidRequestError(
-                "This construct already has multiple parameter sets.")
+                "This construct already has multiple parameter sets."
+            )
 
         if args:
             if len(args) > 1:
                 raise exc.ArgumentError(
                     "Only a single dictionary/tuple or list of "
-                    "dictionaries/tuples is accepted positionally.")
+                    "dictionaries/tuples is accepted positionally."
+                )
             v = args[0]
         else:
             v = {}
 
         if self.parameters is None:
-            self.parameters, self._has_multi_parameters = \
-                self._process_colparams(v)
+            (
+                self.parameters,
+                self._has_multi_parameters,
+            ) = self._process_colparams(v)
         else:
             if self._has_multi_parameters:
                 self.parameters = list(self.parameters)
@@ -356,7 +386,8 @@ class ValuesBase(UpdateBase):
                 if not self._has_multi_parameters:
                     raise exc.ArgumentError(
                         "Can't mix single-values and multiple values "
-                        "formats in one statement")
+                        "formats in one statement"
+                    )
 
                 self.parameters.extend(p)
             else:
@@ -365,14 +396,16 @@ class ValuesBase(UpdateBase):
                 if self._has_multi_parameters:
                     raise exc.ArgumentError(
                         "Can't mix single-values and multiple values "
-                        "formats in one statement")
+                        "formats in one statement"
+                    )
                 self.parameters.update(p)
 
         if kwargs:
             if self._has_multi_parameters:
                 raise exc.ArgumentError(
                     "Can't pass kwargs and multiple parameter sets "
-                    "simultaneously")
+                    "simultaneously"
+                )
             else:
                 self.parameters.update(kwargs)
 
@@ -394,8 +427,10 @@ class ValuesBase(UpdateBase):
         added to any existing RETURNING clause, provided that
         :meth:`.UpdateBase.returning` is not used simultaneously.  The column
         values will then be available on the result using the
-        :attr:`.ResultProxy.returned_defaults` accessor as a dictionary,
-        referring to values keyed to the :class:`.Column` object as well as
+        :attr:`_engine.ResultProxy.returned_defaults` accessor as
+        a dictionary,
+        referring to values keyed to the :class:`_schema.Column`
+        object as well as
         its ``.key``.
 
         This method differs from :meth:`.UpdateBase.returning` in these ways:
@@ -423,13 +458,14 @@ class ValuesBase(UpdateBase):
         3. It can be called against any backend.  Backends that don't support
            RETURNING will skip the usage of the feature, rather than raising
            an exception.  The return value of
-           :attr:`.ResultProxy.returned_defaults` will be ``None``
+           :attr:`_engine.ResultProxy.returned_defaults` will be ``None``
 
         :meth:`.ValuesBase.return_defaults` is used by the ORM to provide
         an efficient implementation for the ``eager_defaults`` feature of
         :func:`.mapper`.
 
-        :param cols: optional list of column key names or :class:`.Column`
+        :param cols: optional list of column key names or
+         :class:`_schema.Column`
          objects.  If omitted, all column expressions evaluated on the server
          are added to the returning list.
 
@@ -439,7 +475,7 @@ class ValuesBase(UpdateBase):
 
             :meth:`.UpdateBase.returning`
 
-            :attr:`.ResultProxy.returned_defaults`
+            :attr:`_engine.ResultProxy.returned_defaults`
 
         """
         self._return_defaults = cols or True
@@ -448,41 +484,46 @@ class ValuesBase(UpdateBase):
 class Insert(ValuesBase):
     """Represent an INSERT construct.
 
-    The :class:`.Insert` object is created using the
-    :func:`~.expression.insert()` function.
+    The :class:`_expression.Insert` object is created using the
+    :func:`_expression.insert()` function.
 
     .. seealso::
 
         :ref:`coretutorial_insert_expressions`
 
     """
-    __visit_name__ = 'insert'
+
+    __visit_name__ = "insert"
 
     _supports_multi_parameters = True
 
-    def __init__(self,
-                 table,
-                 values=None,
-                 inline=False,
-                 bind=None,
-                 prefixes=None,
-                 returning=None,
-                 return_defaults=False,
-                 **dialect_kw):
-        """Construct an :class:`.Insert` object.
+    def __init__(
+        self,
+        table,
+        values=None,
+        inline=False,
+        bind=None,
+        prefixes=None,
+        returning=None,
+        return_defaults=False,
+        **dialect_kw
+    ):
+        """Construct an :class:`_expression.Insert` object.
 
         Similar functionality is available via the
-        :meth:`~.TableClause.insert` method on
-        :class:`~.schema.Table`.
+        :meth:`_expression.TableClause.insert` method on
+        :class:`_schema.Table`.
 
-        :param table: :class:`.TableClause` which is the subject of the
+        :param table: :class:`_expression.TableClause`
+         which is the subject of the
          insert.
 
         :param values: collection of values to be inserted; see
-         :meth:`.Insert.values` for a description of allowed formats here.
-         Can be omitted entirely; a :class:`.Insert` construct will also
-         dynamically render the VALUES clause at execution time based on
-         the parameters passed to :meth:`.Connection.execute`.
+         :meth:`_expression.Insert.values`
+         for a description of allowed formats here.
+         Can be omitted entirely; a :class:`_expression.Insert` construct
+         will also dynamically render the VALUES clause at execution time
+         based on the parameters passed to :meth:`_engine.Connection.execute`.
 
         :param inline: if True, no attempt will be made to retrieve the
          SQL-generated default values to be provided within the statement;
@@ -526,13 +567,13 @@ class Insert(ValuesBase):
 
     def get_children(self, **kwargs):
         if self.select is not None:
-            return self.select,
+            return (self.select,)
         else:
             return ()
 
     @_generative
     def from_select(self, names, select, include_defaults=True):
-        """Return a new :class:`.Insert` construct which represents
+        """Return a new :class:`_expression.Insert` construct which represents
         an ``INSERT...FROM SELECT`` statement.
 
         e.g.::
@@ -540,18 +581,21 @@ class Insert(ValuesBase):
             sel = select([table1.c.a, table1.c.b]).where(table1.c.c > 5)
             ins = table2.insert().from_select(['a', 'b'], sel)
 
-        :param names: a sequence of string column names or :class:`.Column`
+        :param names: a sequence of string column names or
+         :class:`_schema.Column`
          objects representing the target columns.
-        :param select: a :func:`.select` construct, :class:`.FromClause`
-         or other construct which resolves into a :class:`.FromClause`,
-         such as an ORM :class:`.Query` object, etc.  The order of
+        :param select: a :func:`_expression.select` construct,
+         :class:`_expression.FromClause`
+         or other construct which resolves into a
+         :class:`_expression.FromClause`,
+         such as an ORM :class:`_query.Query` object, etc.  The order of
          columns returned from this FROM clause should correspond to the
          order of columns sent as the ``names`` parameter;  while this
          is not checked before passing along to the database, the database
          would normally raise an exception if these column lists don't
          correspond.
         :param include_defaults: if True, non-server default values and
-         SQL expressions as specified on :class:`.Column` objects
+         SQL expressions as specified on :class:`_schema.Column` objects
          (as documented in :ref:`metadata_defaults_toplevel`) not
          otherwise specified in the list of names will be rendered
          into the INSERT and SELECT statements, so that these values are also
@@ -561,28 +605,30 @@ class Insert(ValuesBase):
             will only be invoked **once** for the whole statement, and **not
             per row**.
 
-         .. versionadded:: 1.0.0 - :meth:`.Insert.from_select` now renders
+         .. versionadded:: 1.0.0 - :meth:`_expression.Insert.from_select`
+            now renders
             Python-side and SQL expression column defaults into the
             SELECT statement for columns otherwise not included in the
             list of column names.
 
         .. versionchanged:: 1.0.0 an INSERT that uses FROM SELECT
-           implies that the :paramref:`.insert.inline` flag is set to
+           implies that the :paramref:`_expression.insert.inline`
+           flag is set to
            True, indicating that the statement will not attempt to fetch
            the "last inserted primary key" or other defaults.  The statement
            deals with an arbitrary number of rows, so the
-           :attr:`.ResultProxy.inserted_primary_key` accessor does not apply.
-
-        .. versionadded:: 0.8.3
+           :attr:`_engine.ResultProxy.inserted_primary_key`
+           accessor does not apply.
 
         """
         if self.parameters:
             raise exc.InvalidRequestError(
-                "This construct already inserts value expressions")
+                "This construct already inserts value expressions"
+            )
 
-        self.parameters, self._has_multi_parameters = \
-            self._process_colparams(
-                {_column_as_key(n): Null() for n in names})
+        self.parameters, self._has_multi_parameters = self._process_colparams(
+            {_column_as_key(n): Null() for n in names}
+        )
 
         self.select_names = names
         self.inline = True
@@ -599,24 +645,28 @@ class Insert(ValuesBase):
 class Update(ValuesBase):
     """Represent an Update construct.
 
-    The :class:`.Update` object is created using the :func:`update()`
+    The :class:`_expression.Update`
+    object is created using the :func:`update()`
     function.
 
     """
-    __visit_name__ = 'update'
 
-    def __init__(self,
-                 table,
-                 whereclause=None,
-                 values=None,
-                 inline=False,
-                 bind=None,
-                 prefixes=None,
-                 returning=None,
-                 return_defaults=False,
-                 preserve_parameter_order=False,
-                 **dialect_kw):
-        r"""Construct an :class:`.Update` object.
+    __visit_name__ = "update"
+
+    def __init__(
+        self,
+        table,
+        whereclause=None,
+        values=None,
+        inline=False,
+        bind=None,
+        prefixes=None,
+        returning=None,
+        return_defaults=False,
+        preserve_parameter_order=False,
+        **dialect_kw
+    ):
+        r"""Construct an :class:`_expression.Update` object.
 
         E.g.::
 
@@ -626,14 +676,15 @@ class Update(ValuesBase):
                     values(name='user #5')
 
         Similar functionality is available via the
-        :meth:`~.TableClause.update` method on
-        :class:`.Table`::
+        :meth:`_expression.TableClause.update` method on
+        :class:`_schema.Table`::
 
             stmt = users.update().\
                         where(users.c.id==5).\
                         values(name='user #5')
 
-        :param table: A :class:`.Table` object representing the database
+        :param table: A :class:`_schema.Table`
+         object representing the database
          table to be updated.
 
         :param whereclause: Optional SQL expression describing the ``WHERE``
@@ -655,9 +706,6 @@ class Update(ValuesBase):
                                 as_scalar()
                     )
 
-         .. versionchanged:: 0.7.4
-             The WHERE clause of UPDATE can refer to multiple tables.
-
         :param values:
           Optional dictionary which specifies the ``SET`` conditions of the
           ``UPDATE``.  If left as ``None``, the ``SET``
@@ -667,39 +715,40 @@ class Update(ValuesBase):
           the ``SET`` clause generates for all columns.
 
           Modern applications may prefer to use the generative
-          :meth:`.Update.values` method to set the values of the
+          :meth:`_expression.Update.values` method to set the values of the
           UPDATE statement.
 
         :param inline:
-          if True, SQL defaults present on :class:`.Column` objects via
+          if True, SQL defaults present on :class:`_schema.Column` objects via
           the ``default`` keyword will be compiled 'inline' into the statement
           and not pre-executed.  This means that their values will not
           be available in the dictionary returned from
-          :meth:`.ResultProxy.last_updated_params`.
+          :meth:`_engine.ResultProxy.last_updated_params`.
 
         :param preserve_parameter_order: if True, the update statement is
-          expected to receive parameters **only** via the :meth:`.Update.values`
-          method, and they must be passed as a Python ``list`` of 2-tuples.
-          The rendered UPDATE statement will emit the SET clause for each
-          referenced column maintaining this order.
+          expected to receive parameters **only** via the
+          :meth:`_expression.Update.values` method,
+          and they must be passed as a Python
+          ``list`` of 2-tuples. The rendered UPDATE statement will emit the SET
+          clause for each referenced column maintaining this order.
 
           .. versionadded:: 1.0.10
 
           .. seealso::
 
             :ref:`updates_order_parameters` - full example of the
-            :paramref:`~sqlalchemy.sql.expression.update.preserve_parameter_order` flag
+            :paramref:`_expression.update.preserve_parameter_order` flag
 
         If both ``values`` and compile-time bind parameters are present, the
         compile-time bind parameters override the information specified
         within ``values`` on a per-key basis.
 
-        The keys within ``values`` can be either :class:`.Column`
+        The keys within ``values`` can be either :class:`_schema.Column`
         objects or their string identifiers (specifically the "key" of the
-        :class:`.Column`, normally but not necessarily equivalent to
+        :class:`_schema.Column`, normally but not necessarily equivalent to
         its "name").  Normally, the
-        :class:`.Column` objects used here are expected to be
-        part of the target :class:`.Table` that is the table
+        :class:`_schema.Column` objects used here are expected to be
+        part of the target :class:`_schema.Table` that is the table
         to be updated.  However when using MySQL, a multiple-table
         UPDATE statement can refer to columns from any of
         the tables referred to in the WHERE clause.
@@ -707,13 +756,13 @@ class Update(ValuesBase):
         The values referred to in ``values`` are typically:
 
         * a literal data value (i.e. string, number, etc.)
-        * a SQL expression, such as a related :class:`.Column`,
-          a scalar-returning :func:`.select` construct,
+        * a SQL expression, such as a related :class:`_schema.Column`,
+          a scalar-returning :func:`_expression.select` construct,
           etc.
 
-        When combining :func:`.select` constructs within the values
-        clause of an :func:`.update` construct,
-        the subquery represented by the :func:`.select` should be
+        When combining :func:`_expression.select` constructs within the values
+        clause of an :func:`_expression.update` construct,
+        the subquery represented by the :func:`_expression.select` should be
         *correlated* to the parent table, that is, providing criterion
         which links the table inside the subquery to the outer table
         being updated::
@@ -745,7 +794,7 @@ class Update(ValuesBase):
 
     def get_children(self, **kwargs):
         if self._whereclause is not None:
-            return self._whereclause,
+            return (self._whereclause,)
         else:
             return ()
 
@@ -756,13 +805,14 @@ class Update(ValuesBase):
 
     @_generative
     def where(self, whereclause):
-        """return a new update() construct with the given expression added to
+        """Return a new update() construct with the given expression added to
         its WHERE clause, joined to the existing clause via AND, if any.
 
         """
         if self._whereclause is not None:
-            self._whereclause = and_(self._whereclause,
-                                     _literal_as_text(whereclause))
+            self._whereclause = and_(
+                self._whereclause, _literal_as_text(whereclause)
+            )
         else:
             self._whereclause = _literal_as_text(whereclause)
 
@@ -783,29 +833,33 @@ class Update(ValuesBase):
 class Delete(UpdateBase):
     """Represent a DELETE construct.
 
-    The :class:`.Delete` object is created using the :func:`delete()`
+    The :class:`_expression.Delete`
+    object is created using the :func:`delete()`
     function.
 
     """
 
-    __visit_name__ = 'delete'
+    __visit_name__ = "delete"
 
-    def __init__(self,
-                 table,
-                 whereclause=None,
-                 bind=None,
-                 returning=None,
-                 prefixes=None,
-                 **dialect_kw):
-        """Construct :class:`.Delete` object.
+    def __init__(
+        self,
+        table,
+        whereclause=None,
+        bind=None,
+        returning=None,
+        prefixes=None,
+        **dialect_kw
+    ):
+        r"""Construct :class:`_expression.Delete` object.
 
         Similar functionality is available via the
-        :meth:`~.TableClause.delete` method on
-        :class:`~.schema.Table`.
+        :meth:`_expression.TableClause.delete` method on
+        :class:`_schema.Table`.
 
         :param table: The table to delete rows from.
 
-        :param whereclause: A :class:`.ClauseElement` describing the ``WHERE``
+        :param whereclause: A :class:`_expression.ClauseElement`
+          describing the ``WHERE``
           condition of the ``DELETE`` statement. Note that the
           :meth:`~Delete.where()` generative method may be used instead.
 
@@ -847,7 +901,7 @@ class Delete(UpdateBase):
 
     def get_children(self, **kwargs):
         if self._whereclause is not None:
-            return self._whereclause,
+            return (self._whereclause,)
         else:
             return ()
 
@@ -856,8 +910,9 @@ class Delete(UpdateBase):
         """Add the given WHERE clause to a newly returned delete construct."""
 
         if self._whereclause is not None:
-            self._whereclause = and_(self._whereclause,
-                                     _literal_as_text(whereclause))
+            self._whereclause = and_(
+                self._whereclause, _literal_as_text(whereclause)
+            )
         else:
             self._whereclause = _literal_as_text(whereclause)
 
